# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/store/shop_game_info.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_shop_info(object):
    def setupUi(self, shop_info):
        shop_info.setObjectName("shop_info")
        shop_info.resize(702, 468)
        shop_info.setWindowTitle("Form")
        self.verticalLayout = QtWidgets.QVBoxLayout(shop_info)
        self.verticalLayout.setObjectName("verticalLayout")
        self.back_button = QtWidgets.QPushButton(shop_info)
        self.back_button.setObjectName("back_button")
        self.verticalLayout.addWidget(self.back_button)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.image_stack = QtWidgets.QStackedWidget(shop_info)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.image_stack.sizePolicy().hasHeightForWidth())
        self.image_stack.setSizePolicy(sizePolicy)
        self.image_stack.setObjectName("image_stack")
        self.horizontalLayout.addWidget(self.image_stack)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.title = QtWidgets.QLabel(shop_info)
        font = QtGui.QFont()
        font.setPointSize(18)
        self.title.setFont(font)
        self.title.setText("")
        self.title.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse | QtCore.Qt.TextSelectableByMouse)
        self.title.setObjectName("title")
        self.verticalLayout_2.addWidget(self.title)
        self.dev = QtWidgets.QLabel(shop_info)
        font = QtGui.QFont()
        font.setPointSize(14)
        self.dev.setFont(font)
        self.dev.setText("")
        self.dev.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse | QtCore.Qt.TextSelectableByMouse)
        self.dev.setObjectName("dev")
        self.verticalLayout_2.addWidget(self.dev)
        self.owned_label = QtWidgets.QLabel(shop_info)
        self.owned_label.setObjectName("owned_label")
        self.verticalLayout_2.addWidget(self.owned_label)
        self.price = QtWidgets.QLabel(shop_info)
        self.price.setText("")
        self.price.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse | QtCore.Qt.TextSelectableByMouse)
        self.price.setObjectName("price")
        self.verticalLayout_2.addWidget(self.price)
        self.discount_price = QtWidgets.QLabel(shop_info)
        self.discount_price.setText("")
        self.discount_price.setObjectName("discount_price")
        self.verticalLayout_2.addWidget(self.discount_price)
        self.tags = QtWidgets.QLabel(shop_info)
        self.tags.setText("")
        self.tags.setObjectName("tags")
        self.verticalLayout_2.addWidget(self.tags)
        self.open_store_button = QtWidgets.QPushButton(shop_info)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.open_store_button.sizePolicy().hasHeightForWidth())
        self.open_store_button.setSizePolicy(sizePolicy)
        self.open_store_button.setObjectName("open_store_button")
        self.verticalLayout_2.addWidget(self.open_store_button)
        self.wishlist_button = QtWidgets.QPushButton(shop_info)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.wishlist_button.sizePolicy().hasHeightForWidth())
        self.wishlist_button.setSizePolicy(sizePolicy)
        self.wishlist_button.setObjectName("wishlist_button")
        self.verticalLayout_2.addWidget(self.wishlist_button)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem)
        self.horizontalLayout.addLayout(self.verticalLayout_2)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.req_group_box = QtWidgets.QGroupBox(shop_info)
        self.req_group_box.setObjectName("req_group_box")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.req_group_box)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.verticalLayout.addWidget(self.req_group_box)
        self.social_link_gb = QtWidgets.QGroupBox(shop_info)
        self.social_link_gb.setObjectName("social_link_gb")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.social_link_gb)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.verticalLayout.addWidget(self.social_link_gb)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem2)

        self.retranslateUi(shop_info)
        self.image_stack.setCurrentIndex(-1)

    def retranslateUi(self, shop_info):
        _translate = QtCore.QCoreApplication.translate
        self.back_button.setText(_translate("shop_info", "Back"))
        self.owned_label.setText(_translate("shop_info", "You already own this game"))
        self.open_store_button.setText(_translate("shop_info", "Buy Game in Epic Games Store"))
        self.wishlist_button.setText(_translate("shop_info", "Add to wishlist"))
        self.req_group_box.setTitle(_translate("shop_info", "Requirements"))
        self.social_link_gb.setTitle(_translate("shop_info", "Social Links"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    shop_info = QtWidgets.QWidget()
    ui = Ui_shop_info()
    ui.setupUi(shop_info)
    shop_info.show()
    sys.exit(app.exec_())



from dataclasses import dataclass
from typing import Dict


class Actions:
    undefined = 0
    finished = 1
    error = 2
    message = 3
    state_update = 4


@dataclass
class BaseModel:
    action: int
    app_name: str

    @classmethod
    def from_json(cls, data: Dict):
        return cls(
            action=data["action"],
            app_name=data["app_name"]
        )


@dataclass
class FinishedModel(BaseModel):
    exit_code: int
    playtime: int  # seconds

    @classmethod
    def from_json(cls, data: Dict):
        return cls(
            **vars(BaseModel.from_json(data)),
            exit_code=data["exit_code"],
            playtime=data["playtime"],
        )


@dataclass
class StateChangedModel(BaseModel):
    class States:
        started = 1
        # for future
        syncing_cloud = 2
        cloud_sync_finished = 3

    new_state: int

    @classmethod
    def from_json(cls, data: Dict):
        return cls(
            action=data["action"],
            app_name=data["app_name"],
            new_state=data["new_state"]
        )


@dataclass
class ErrorModel(BaseModel):
    error_string: str

    @classmethod
    def from_json(cls, data: Dict):
        return cls(
            **vars(BaseModel.from_json(data)),
            error_string=data["error_string"]
        )



import os
from enum import IntEnum
from logging import getLogger
from typing import List, Union, Type, Dict

import qtawesome
import requests
from PyQt5.QtCore import (
    QObject,
    QSettings,
    QFile,
    QDir,
    Qt,
)
from PyQt5.QtGui import QPalette, QColor, QFontMetrics
from PyQt5.QtWidgets import qApp, QStyleFactory, QLabel
from PyQt5.sip import wrappertype

from rare.utils.paths import resources_path

logger = getLogger("Utils")


class ExitCodes(IntEnum):
    EXIT = 0
    LOGOUT = -133742


color_role_map: Dict[int, str] = {
    0: "WindowText",
    1: "Button",
    2: "Light",
    3: "Midlight",
    4: "Dark",
    5: "Mid",
    6: "Text",
    7: "BrightText",
    8: "ButtonText",
    9: "Base",
    10: "Window",
    11: "Shadow",
    12: "Highlight",
    13: "HighlightedText",
    14: "Link",
    15: "LinkVisited",
    16: "AlternateBase",
    # 17: "NoRole",
    18: "ToolTipBase",
    19: "ToolTipText",
    20: "PlaceholderText",
    # 21: "NColorRoles",
}

color_group_map: Dict[int, str] = {
    0: "Active",
    1: "Disabled",
    2: "Inactive",
}


def load_color_scheme(path: str) -> QPalette:
    palette = QPalette()
    scheme = QSettings(path, QSettings.IniFormat)
    try:
        scheme.beginGroup("ColorScheme")
        for g in color_group_map:
            scheme.beginGroup(color_group_map[g])
            group = QPalette.ColorGroup(g)
            for r in color_role_map:
                role = QPalette.ColorRole(r)
                color = scheme.value(color_role_map[r], None)
                if color is not None:
                    palette.setColor(group, role, QColor(color))
                else:
                    palette.setColor(group, role, palette.color(QPalette.Active, role))
            scheme.endGroup()
        scheme.endGroup()
    except:
        palette = None
    return palette


def get_static_style() -> str:
    file = QFile(":/static_css/stylesheet.qss")
    file.open(QFile.ReadOnly)
    static = file.readAll().data().decode("utf-8")
    file.close()
    return static


def set_color_pallete(color_scheme: str) -> None:
    static = get_static_style()

    if not color_scheme:
        qApp.setStyle(QStyleFactory.create(qApp.property("rareDefaultQtStyle")))
        qApp.setPalette(qApp.style().standardPalette())
        qApp.setStyleSheet(static)
        return

    qApp.setStyle(QStyleFactory.create("Fusion"))
    custom_palette = load_color_scheme(f":/schemes/{color_scheme}")
    if custom_palette is not None:
        qApp.setPalette(custom_palette)
        qApp.setStyleSheet(static)
        icon_color = qApp.palette().color(QPalette.Foreground).name()
        qtawesome.set_defaults(color=icon_color)


def get_color_schemes() -> List[str]:
    colors = []
    for file in QDir(":/schemes"):
        colors.append(file)
    return colors


def set_style_sheet(style_sheet: str) -> None:
    static = get_static_style()

    if not style_sheet:
        qApp.setStyle(QStyleFactory.create(qApp.property("rareDefaultQtStyle")))
        qApp.setStyleSheet(static)
        return

    qApp.setStyle(QStyleFactory.create("Fusion"))
    file = QFile(f":/stylesheets/{style_sheet}/stylesheet.qss")
    file.open(QFile.ReadOnly)
    stylesheet = file.readAll().data().decode("utf-8")
    file.close()
    qApp.setStyleSheet(stylesheet + static)

    icon_color = qApp.palette().color(QPalette.Text).name()
    qtawesome.set_defaults(color="#eeeeee")


def get_style_sheets() -> List[str]:
    styles = []
    for file in QDir(":/stylesheets/"):
        styles.append(file)
    return styles


def get_translations():
    langs = ["en"]
    for i in os.listdir(os.path.join(resources_path, "languages")):
        if i.endswith(".qm") and not i.startswith("qt_"):
            langs.append(i.split(".")[0])
    return langs


def get_latest_version():
    try:
        resp = requests.get(
            "https://api.github.com/repos/RareDevs/Rare/releases/latest",
            timeout=2,
        )
        tag = resp.json()["tag_name"]
        return tag
    except requests.exceptions.ConnectionError:
        return "0.0.0"


def path_size(path: Union[str, os.PathLike]) -> int:
    return sum(
        os.stat(os.path.join(dp, f)).st_size
        for dp, dn, filenames in os.walk(path)
        for f in filenames
        if os.path.isfile(os.path.join(dp, f))
    )


def format_size(b: Union[int, float]) -> str:
    for s in ["", "Ki", "Mi", "Gi", "Ti", "Pi", "Ei"]:
        if b < 1024:
            return f"{b:.2f} {s}B"
        b /= 1024


def icon(icn_str: str, fallback: str = None, **kwargs):
    try:
        return qtawesome.icon(icn_str, **kwargs)
    except Exception as e:
        if not fallback:
            logger.warning(f"{e} {icn_str}")
    if fallback:
        try:
            return qtawesome.icon(fallback, **kwargs)
        except Exception as e:
            logger.error(str(e))
    if kwargs.get("color"):
        kwargs["color"] = "red"
    return qtawesome.icon("ei.error", **kwargs)


def widget_object_name(widget: Union[QObject, wrappertype, Type], suffix: str) -> str:
    suffix = f"_{suffix}" if suffix else ""
    if isinstance(widget, QObject):
        return f"{type(widget).__name__}{suffix}"
    elif isinstance(widget, wrappertype) or isinstance(widget, type):
        return f"{widget.__name__}{suffix}"
    else:
        raise RuntimeError(f"Argument {widget} not a QObject or type of QObject")


def elide_text(label: QLabel, text: str) -> str:
    metrics = QFontMetrics(label.font())
    return metrics.elidedText(text, Qt.ElideRight, label.sizeHint().width())



import setuptools

from rare import __version__ as version

with open("README.md", "r") as fh:
    long_description = fh.read()

requirements = [
    "requests<3.0",
    "legendary-gl>=0.20.34",
    "orjson",
    "setuptools",
    "wheel",
    "PyQt5",
    "QtAwesome",
    'pywin32; platform_system == "Windows"',
]

optional_reqs = dict(
    webview=[
        'pywebview[gtk]; platform_system == "Linux"',
        'pywebview[gtk]; platform_system == "FreeBSD"',
        'pywebview[cef]; platform_system == "Windows"',
    ],
    pypresence=["pypresence"]
)

setuptools.setup(
    name="Rare",
    version=version,
    author="Dummerle",
    license="GPL-3",
    description="A gui for Legendary",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Dummerle/Rare",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        "Operating System :: OS Independent"
    ],
    include_package_data=True,
    python_requires=">=3.9",
    entry_points={
        # 'console_scripts': ["rare = rare.main:main"],
        'gui_scripts': ["rare = rare.main:main"],
    },
    install_requires=requirements,
    extras_require=optional_reqs,
)



from dataclasses import dataclass, field
from enum import IntEnum
from typing import List, Optional, Dict

from rare.lgndr.glue.monkeys import (
    LgndrIndirectStatus,
    GetBooleanChoiceProtocol,
    SdlPromptProtocol,
    VerifyStdoutProtocol,
    UiUpdateProtocol,
    get_boolean_choice_factory,
    sdl_prompt_factory,
    verify_stdout_factory,
    ui_update_factory,
    DLManagerSignals,
)

"""
@dataclass(kw_only=True)
class LgndrCommonArgs:
    # keep this here for future reference
    # when we move to 3.10 we can use 'kw_only' to do dataclass inheritance
    app_name: str
    platform: str = "Windows"
    yes: bool = False
"""


@dataclass
class LgndrImportGameArgs:
    app_path: str
    app_name: str
    platform: str = "Windows"
    disable_check: bool = False
    skip_dlcs: bool = False
    with_dlcs: bool = False
    yes: bool = False
    # Rare: Extra arguments
    indirect_status: LgndrIndirectStatus = field(default_factory=LgndrIndirectStatus)
    get_boolean_choice: GetBooleanChoiceProtocol = get_boolean_choice_factory(True)


@dataclass
class LgndrUninstallGameArgs:
    app_name: str
    keep_files: bool = False
    skip_uninstaller: bool = False
    yes: bool = False
    # Rare: Extra arguments
    indirect_status: LgndrIndirectStatus = field(default_factory=LgndrIndirectStatus)
    get_boolean_choice_main: GetBooleanChoiceProtocol = get_boolean_choice_factory(True)
    get_boolean_choice_handler: GetBooleanChoiceProtocol = get_boolean_choice_factory(True)


@dataclass
class LgndrVerifyGameArgs:
    app_name: str
    # Rare: Extra arguments
    indirect_status: LgndrIndirectStatus = field(default_factory=LgndrIndirectStatus)
    verify_stdout: VerifyStdoutProtocol = verify_stdout_factory(None)


@dataclass
class LgndrInstallGameArgs:
    app_name: str
    base_path: str = ""
    shared_memory: int = 0
    max_workers: int = 0
    force: bool = False
    disable_patching: bool = False
    game_folder: str = ""
    override_manifest: str = ""
    override_old_manifest: str = ""
    override_base_url: str = ""
    platform: str = "Windows"
    file_prefix: List = None
    file_exclude_prefix: List = None
    install_tag: Optional[List[str]] = None
    order_opt: bool = False
    dl_timeout: int = 10
    repair_mode: bool = False
    repair_and_update: bool = False
    disable_delta: bool = False
    override_delta_manifest: str = ""
    egl_guid: str = ""
    preferred_cdn: str = None
    no_install: bool = False
    ignore_space: bool = False
    disable_sdl: bool = False
    reset_sdl: bool = False
    skip_sdl: bool = False
    disable_https: bool = False
    bind_ip: str = ""
    # FIXME: move to LgndrInstallGameRealArgs
    skip_dlcs: bool = False
    with_dlcs: bool = False
    # end of FIXME
    yes: bool = True
    # Rare: Extra arguments
    indirect_status: LgndrIndirectStatus = field(default_factory=LgndrIndirectStatus)
    get_boolean_choice: GetBooleanChoiceProtocol = get_boolean_choice_factory(True)
    verify_stdout: VerifyStdoutProtocol = verify_stdout_factory(None)

    def __post_init__(self):
        self.sdl_prompt: SdlPromptProtocol = sdl_prompt_factory(self.install_tag)


@dataclass
class LgndrInstallGameRealArgs:
    app_name: str
    platform: str = "Windows"
    repair_mode: bool = False
    repair_file: str = ""
    no_install: bool = False
    save_path: str = ""
    skip_dlcs: bool = False
    with_dlcs: bool = False
    dlm_debug: bool = False
    yes: bool = False
    # Rare: Extra arguments
    install_prereqs: bool = False
    indirect_status: LgndrIndirectStatus = field(default_factory=LgndrIndirectStatus)
    ui_update: UiUpdateProtocol = ui_update_factory(None)
    dlm_signals: DLManagerSignals = field(default_factory=DLManagerSignals)


@dataclass
class LgndrInstallGameRealRet:
    class ReturnCode(IntEnum):
        ERROR = 1
        STOPPED = 2
        FINISHED = 3

    app_name: str
    ret_code: ReturnCode = ReturnCode.ERROR
    message: str = ""
    dlcs: Optional[List[Dict]] = None
    sync_saves: bool = False
    tip_url: str = ""
    shortcuts: bool = False



from enum import Enum
from typing import Tuple, Optional, Union

from PyQt5.QtCore import Qt, QRectF, QSize
from PyQt5.QtGui import (
    QPaintEvent,
    QPainter,
    QPixmap,
    QTransform,
    QBrush,
    QPalette,
    QPainterPath,
    QLinearGradient,
    QColor,
)
from PyQt5.QtWidgets import QWidget

from rare.models.image import ImageSize

OverlayPath = Tuple[QPainterPath, Union[QColor, QLinearGradient]]


class ImageWidget(QWidget):
    class Border(Enum):
        Rounded = 0
        Squared = 1

    _pixmap: Optional[QPixmap] = None
    _opacity: float = 1.0
    _transform: QTransform
    _smooth_transform: bool = False
    _rounded_overlay: Optional[OverlayPath] = None
    _squared_overlay: Optional[OverlayPath] = None
    _image_size: Optional[ImageSize.Preset] = None

    def __init__(self, parent=None) -> None:
        super(ImageWidget, self).__init__(parent=parent)
        self.setObjectName(type(self).__name__)
        self.setContentsMargins(0, 0, 0, 0)
        self.paint_image = self.paint_image_empty
        self.paint_overlay = self.paint_overlay_rounded

    def setOpacity(self, value: float) -> None:
        self._opacity = value
        self.update()

    def setPixmap(self, pixmap: QPixmap) -> None:
        if not pixmap.isNull():
            self._pixmap = pixmap
            self.paint_image = self.paint_image_cover
            if not self._image_size:
                self._transform = QTransform().scale(
                    1 / pixmap.devicePixelRatioF(),
                    1 / pixmap.devicePixelRatioF(),
                )
            else:
                self._transform = QTransform().scale(
                    1 / pixmap.devicePixelRatioF() / self._image_size.divisor,
                    1 / pixmap.devicePixelRatioF() / self._image_size.divisor,
                )
        else:
            self.paint_image = self.paint_image_empty
        self.update()

    def sizeHint(self) -> QSize:
        return self._image_size.size if self._image_size else super(ImageWidget, self).sizeHint()

    def minimumSizeHint(self) -> QSize:
        return self._image_size.size if self._image_size else super(ImageWidget, self).minimumSizeHint()

    def setFixedSize(self, a0: ImageSize.Preset) -> None:
        self._squared_overlay = None
        self._rounded_overlay = None
        self._image_size = a0
        self._smooth_transform = a0.smooth
        super(ImageWidget, self).setFixedSize(a0.size)

    def setBorder(self, border: Border):
        if border == ImageWidget.Border.Rounded:
            self.paint_overlay = self.paint_overlay_rounded
        else:
            self.paint_overlay = self.paint_overlay_squared
        self.update()

    def _generate_squared_overlay(self) -> OverlayPath:
        if self._image_size is not None and self._squared_overlay is not None:
            return self._squared_overlay
        path = QPainterPath()
        path.addRect(0, 0, self.width(), self.height())
        border = 2
        inner_path = QPainterPath()
        inner_path.addRect(
            QRectF(
                border,
                border,
                self.width() - border * 2,
                self.height() - border * 2,
            )
        )
        gradient = QLinearGradient(0, 0, 0, self.height())
        gradient.setColorAt(0.0, Qt.black)
        gradient.setColorAt(1.0, Qt.transparent)
        self._squared_overlay = path.subtracted(inner_path), gradient
        return self._squared_overlay

    def _generate_rounded_overlay(self) -> OverlayPath:
        if self._image_size is not None and self._rounded_overlay is not None:
            return self._rounded_overlay
        # lk: the '-1' and '+1' are adjustments required for anti-aliasing
        # lk: otherwise vertical lines would appear at the edges
        path = QPainterPath()
        path.addRect(-1, -1, self.width() + 2, self.height() + 2)
        rounded_path = QPainterPath()
        rounded_path.addRoundedRect(
            QRectF(0, 0, self.width(), self.height()),
            self.height() * 0.045,
            self.height() * 0.045,
        )
        self._rounded_overlay = path.subtracted(rounded_path), QColor(0, 0, 0, 0)
        return self._rounded_overlay

    def paint_image_empty(self, painter: QPainter, a0: QPaintEvent) -> None:
        # when pixmap object is not available yet, show a gray rectangle
        painter.setOpacity(0.5 * self._opacity)
        painter.fillRect(a0.rect(), Qt.darkGray)

    def paint_image_cover(self, painter: QPainter, a0: QPaintEvent) -> None:
        painter.setOpacity(self._opacity)
        brush = QBrush(self._pixmap)
        # downscale the image during painting to fit the pixelratio
        brush.setTransform(self._transform)
        painter.fillRect(a0.rect(), brush)

    def paint_overlay_rounded(self, painter: QPainter, a0: QPaintEvent) -> None:
        painter.setRenderHint(QPainter.Antialiasing, True)
        painter.setOpacity(1.0)
        painter.setCompositionMode(QPainter.CompositionMode_Source)
        overlay, _ = self._generate_rounded_overlay()
        painter.fillPath(overlay, self.palette().color(QPalette.Background))

    def paint_overlay_squared(self, painter: QPainter, a0: QPaintEvent) -> None:
        painter.setRenderHint(QPainter.Antialiasing, False)
        painter.setOpacity(self._opacity)
        painter.fillPath(*self._generate_squared_overlay())

    def paintEvent(self, a0: QPaintEvent) -> None:
        painter = QPainter(self)
        if not painter.paintEngine().isActive():
            return
        # helps with better image quality
        painter.setRenderHint(QPainter.SmoothPixmapTransform, self._smooth_transform)
        self.paint_image(painter, a0)
        self.paint_overlay(painter, a0)
        painter.end()


__all__ = ["ImageSize", "ImageWidget"]




import urllib.parse
from logging import getLogger

from PyQt5.QtCore import pyqtSignal, QObject

from rare.components.tabs.store.constants import (
    wishlist_query,
    search_query,
    add_to_wishlist_query,
    remove_from_wishlist_query,
)
from rare.components.tabs.store.shop_models import BrowseModel
from rare.utils.qt_requests import QtRequests

logger = getLogger("ShopAPICore")
graphql_url = "https://www.epicgames.com/graphql"


class ShopApiCore(QObject):
    update_wishlist = pyqtSignal()

    def __init__(self, auth_token, lc: str, cc: str):
        super(ShopApiCore, self).__init__()
        self.token = auth_token
        self.language_code: str = lc
        self.country_code: str = cc
        self.locale = f"{self.language_code}-{self.country_code}"
        self.manager = QtRequests(parent=self)
        self.auth_manager = QtRequests(token=auth_token, parent=self)

        self.browse_active = False
        self.next_browse_request = tuple(())

    def get_free_games(self, handle_func: callable):
        url = f"https://store-site-backend-static-ipv4.ak.epicgames.com/freeGamesPromotions?locale={self.language_code}&country={self.country_code}&allowCountries={self.country_code}"

        self.manager.get(url, lambda data: self._handle_free_games(data, handle_func))

    def _handle_free_games(self, data, handle_func):
        try:
            results: dict = data["data"]["Catalog"]["searchStore"]["elements"]
        except KeyError:
            logger.error("Free games Api request failed")
            handle_func(["error", "Key error"])
            return
        except Exception as e:
            logger.error(f"Free games Api request failed: {e}")
            handle_func(["error", e])
            return
        handle_func(results)

    def get_wishlist(self, handle_func):
        self.auth_manager.post(
            graphql_url,
            lambda data: self._handle_wishlist(data, handle_func),
            {
                "query": wishlist_query,
                "variables": {
                    "country": self.country_code,
                    "locale": f"{self.language_code}-{self.country_code}",
                },
            },
        )

    def _handle_wishlist(self, data, handle_func):
        try:
            results: list = data["data"]["Wishlist"]["wishlistItems"]["elements"]
        except KeyError:
            logger.error("Free games Api request failed")
            handle_func(["error", "Key error"])
            return
        except Exception as e:
            logger.error(f"Free games Api request failed: {e}")
            handle_func(["error", e])
            return

        handle_func(results)

    def search_game(self, name, handle_func):
        payload = {
            "query": search_query,
            "variables": {
                "category": "games/edition/base|bundles/games|editors|software/edition/base",
                "count": 10,
                "country": self.country_code,
                "keywords": name,
                "locale": self.locale,
                "sortDir": "DESC",
                "allowCountries": self.country_code,
                "start": 0,
                "tag": "",
                "withMapping": False,
                "withPrice": True,
            },
        }

        self.manager.post(
            graphql_url, lambda data: self._handle_search(data, handle_func), payload,
        )

    def _handle_search(self, data, handle_func):
        try:
            handle_func(data["data"]["Catalog"]["searchStore"]["elements"])
        except KeyError as e:
            logger.error(str(e))
            handle_func([])
        except Exception as e:
            logger.error(f"Search Api request failed: {e}")
            handle_func([])
            return

    def browse_games(self, browse_model: BrowseModel, handle_func):
        if self.browse_active:
            self.next_browse_request = (browse_model, handle_func)
            return
        self.browse_active = True
        url = "https://www.epicgames.com/graphql?operationName=searchStoreQuery&variables={}&extensions={}"
        variables = urllib.parse.quote_plus(str(
            dict(browse_model.__dict__))
        )
        extensions = urllib.parse.quote_plus(str(
            dict(
                persistedQuery=dict(
                    version=1,
                    sha256Hash="6e7c4dd0177150eb9a47d624be221929582df8648e7ec271c821838ff4ee148e"
                )
            )
        )
        )

        for old, new in [
            ("%26", "&"),
            ("%27", "%22"),
            ("+", ""),
            ("%3A", ":"),
            ("%2C", ","),
            ("%5B", "["),
            ("%5D", "]"),
            ("True", "true"),
        ]:
            variables = variables.replace(old, new)
            extensions = extensions.replace(old, new)

        url = url.format(variables, extensions)
        self.auth_manager.get(
            url, lambda data: self._handle_browse_games(data, handle_func)
        )

    def _handle_browse_games(self, data, handle_func):
        self.browse_active = False
        if data is None:
            data = {}
        if not self.next_browse_request:

            try:
                handle_func(data["data"]["Catalog"]["searchStore"]["elements"])
            except KeyError as e:
                logger.error(str(e))
                handle_func([])
            except Exception as e:
                logger.error(f"Browse games Api request failed: {e}")
                handle_func([])
                return
        else:
            self.browse_games(*self.next_browse_request)  # pylint: disable=E1120
            self.next_browse_request = tuple(())

    def get_game(self, slug: str, is_bundle: bool, handle_func):
        url = f"https://store-content.ak.epicgames.com/api/{self.locale}/content/{'products' if not is_bundle else 'bundles'}/{slug}"
        self.manager.get(url, lambda data: self._handle_get_game(data, handle_func))

    def _handle_get_game(self, data, handle_func):
        try:
            handle_func(data)
        except Exception as e:
            logger.error(str(e))
            handle_func({})

    # needs a captcha
    def add_to_wishlist(self, namespace, offer_id, handle_func: callable):
        payload = {
            "variables": {
                "offerId": offer_id,
                "namespace": namespace,
                "country": self.country_code,
                "locale": self.locale,
            },
            "query": add_to_wishlist_query,
        }
        self.auth_manager.post(
            graphql_url,
            lambda data: self._handle_add_to_wishlist(data, handle_func),
            payload,
        )

    def _handle_add_to_wishlist(self, data, handle_func):
        try:
            data = data["data"]["Wishlist"]["addToWishlist"]
            if data["success"]:
                handle_func(True)
            else:
                handle_func(False)
        except Exception as e:
            logger.error(str(e))
            handle_func(False)
        self.update_wishlist.emit()

    def remove_from_wishlist(self, namespace, offer_id, handle_func: callable):
        payload = {
            "variables": {
                "offerId": offer_id,
                "namespace": namespace,
                "operation": "REMOVE",
            },
            "query": remove_from_wishlist_query,
        }
        self.auth_manager.post(
            graphql_url,
            lambda data: self._handle_remove_from_wishlist(data, handle_func),
            payload,
        )

    def _handle_remove_from_wishlist(self, data, handle_func):
        try:
            data = data["data"]["Wishlist"]["removeFromWishlist"]
            if data["success"]:
                handle_func(True)
            else:
                handle_func(False)
        except Exception as e:
            logger.error(str(e))
            handle_func(False)
        self.update_wishlist.emit()



import os
import shutil
from logging import getLogger

from PyQt5.QtCore import pyqtSignal, QObject
from legendary.lfs.utils import validate_files
from legendary.models.game import VerifyResult

from rare.models.game import RareGame
from rare.lgndr.core import LegendaryCore
from .worker import QueueWorker, QueueWorkerInfo

logger = getLogger("MoveWorker")


class MoveWorker(QueueWorker):
    class Signals(QObject):
        # int: percentage, object: source size, object: dest size
        progress = pyqtSignal(RareGame, int, object, object)
        # str: destination path
        result = pyqtSignal(RareGame, str)
        # str: error message
        error = pyqtSignal(RareGame, str)

    def __init__(self, core: LegendaryCore, rgame: RareGame, dst_path: str, dst_exists: bool):
        super(MoveWorker, self).__init__()
        self.signals = MoveWorker.Signals()
        self.core = core
        self.rgame = rgame
        self.dst_path = dst_path
        self.dst_exists = dst_exists

    def worker_info(self) -> QueueWorkerInfo:
        return QueueWorkerInfo(
            app_name=self.rgame.app_name, app_title=self.rgame.app_title, worker_type="Move", state=self.state
        )

    def progress(self, src_size, dst_size):
        progress = dst_size * 100 // src_size
        self.rgame.signals.progress.update.emit(progress)
        self.signals.progress.emit(self.rgame, progress, src_size, dst_size)

    def run_real(self):
        self.rgame.signals.progress.start.emit()

        if os.stat(self.rgame.install_path).st_dev == os.stat(os.path.dirname(self.dst_path)).st_dev:
            shutil.move(self.rgame.install_path, self.dst_path)

        elif not self.dst_exists:
            src_size = sum(
                os.stat(os.path.join(dp, f)).st_size
                for dp, dn, filenames in os.walk(self.rgame.install_path)
                for f in filenames
            )
            dst_size = 0

            def copy_with_progress(src, dst):
                nonlocal dst_size
                shutil.copy2(src, dst)
                dst_size += os.stat(src).st_size
                self.progress(src_size, dst_size)

            shutil.copytree(
                self.rgame.install_path,
                self.dst_path,
                copy_function=copy_with_progress,
                dirs_exist_ok=True,
            )
            shutil.rmtree(self.rgame.install_path)

        else:
            manifest_data, _ = self.core.get_installed_manifest(self.rgame.app_name)
            manifest = self.core.load_manifest(manifest_data)
            files = sorted(
                manifest.file_manifest_list.elements,
                key=lambda a: a.filename.lower(),
            )
            if config_tags := self.core.lgd.config.get(self.rgame.app_name, 'install_tags', fallback=None):
                install_tags = set(i.strip() for i in config_tags.split(','))
                file_list = [
                    (f.filename, f.sha_hash.hex())
                    for f in files
                    if any(it in install_tags for it in f.install_tags) or not f.install_tags
                ]
            else:
                file_list = [(f.filename, f.sha_hash.hex()) for f in files]

            total_size = sum(manifest.file_manifest_list.get_file_by_path(fm[0]).file_size
                             for fm in file_list)
            dst_size = 0

            # This method is a copy_func, and only copies the src if it's a dir.
            # Thus, it can be used to re-create the dir structure.
            def copy_dir_structure(src, dst):
                if os.path.isdir(dst):
                    dst = os.path.join(dst, os.path.basename(src))
                if os.path.isdir(src):
                    shutil.copyfile(src, dst)
                    shutil.copystat(src, dst)
                return dst

            # recreate dir structure
            shutil.copytree(
                self.rgame.install_path,
                self.dst_path,
                copy_function=copy_dir_structure,
                dirs_exist_ok=True,
            )

            for result, path, _, _ in validate_files(self.dst_path, file_list):
                dst_path = os.path.join(self.dst_path, path)
                src_path = os.path.join(self.rgame.install_path, path)
                if os.path.isfile(src_path):
                    if result == VerifyResult.HASH_MATCH:
                        dst_size += os.stat(dst_path).st_size
                    if result == VerifyResult.HASH_MISMATCH or result == VerifyResult.FILE_MISSING:
                        try:
                            shutil.copy2(src_path, dst_path)
                            dst_size += os.stat(dst_path).st_size
                        except (IOError, OSError) as e:
                            self.rgame.signals.progress.finish.emit(True)
                            self.signals.error.emit(self.rgame, str(e))
                            return
                    else:
                        logger.warning(f"Copying file {src_path} to {dst_path} failed")
                    self.progress(total_size, dst_size)
                else:
                    logger.warning(
                        f"Source dir does not have file {src_path}. File will be missing in the destination dir."
                    )
                    self.rgame.needs_verification = True
            shutil.rmtree(self.rgame.install_path)

        self.rgame.install_path = self.dst_path

        self.rgame.signals.progress.finish.emit(False)
        self.signals.result.emit(self.rgame, self.dst_path)



import os
from enum import IntEnum
from logging import getLogger
from typing import Callable, Tuple, Optional, Dict, List

from PyQt5.QtCore import (
    QSize,
    pyqtSignal,
    QFileInfo,
    QRunnable,
    QObject,
    QThreadPool,
    pyqtSlot,
    QDir,
)
from PyQt5.QtWidgets import (
    QSizePolicy,
    QLabel,
    QFileDialog,
    QHBoxLayout,
    QWidget,
    QLineEdit,
    QToolButton,
    QCompleter,
    QFileSystemModel,
    QStyledItemDelegate,
    QFileIconProvider,
)

from rare.utils.misc import icon as qta_icon

logger = getLogger("IndicatorEdit")


class IndicatorReasonsCommon(IntEnum):
    VALID = 0
    UNDEFINED = 1
    EMPTY = 2
    WRONG_FORMAT = 3
    WRONG_PATH = 4
    DIR_NOT_EMPTY = 5
    DIR_NOT_EXISTS = 6
    FILE_NOT_EXISTS = 7
    NOT_INSTALLED = 8


class IndicatorReasons(IntEnum):
    """
    Empty enumeration with auto-generated enumeration values.
    Extend this class per-case to implement dedicated message types.
    Types should be assigned using `auto()` from enum

    example:
    MyReasons(IndicatorReasons):
        MY_REASON = auto()
    """
    @staticmethod
    def _generate_next_value_(name, start, count, last_values):
        """generate consecutive automatic numbers starting from zero"""
        start = len(IndicatorReasonsCommon)
        return IntEnum._generate_next_value_(name, start, count, last_values)


class IndicatorReasonsStrings(QObject):
    def __init__(self, parent=None):
        super(IndicatorReasonsStrings, self).__init__(parent=parent)
        self.__text = {
            IndicatorReasonsCommon.VALID: self.tr("Ok!"),
            IndicatorReasonsCommon.UNDEFINED: self.tr("Unknown error occurred"),
            IndicatorReasonsCommon.EMPTY: self.tr("Value can not be empty"),
            IndicatorReasonsCommon.WRONG_FORMAT: self.tr("Wrong format"),
            IndicatorReasonsCommon.WRONG_PATH: self.tr("Wrong file or directory"),
            IndicatorReasonsCommon.DIR_NOT_EMPTY: self.tr("Directory is not empty"),
            IndicatorReasonsCommon.DIR_NOT_EXISTS: self.tr("Directory does not exist"),
            IndicatorReasonsCommon.FILE_NOT_EXISTS: self.tr("File does not exist"),
            IndicatorReasonsCommon.NOT_INSTALLED: self.tr("Game is not installed or does not exist"),
        }

    def __getitem__(self, item: int) -> str:
        return self.__text[item]

    def __setitem__(self, key: int, value: str):
        self.__text[key] = value

    def extend(self, reasons: Dict):
        for k in self.__text.keys():
            if k in reasons.keys():
                raise RuntimeError(f"{reasons} contains existing values")
        self.__text.update(reasons)


class EditFuncRunnable(QRunnable):
    class Signals(QObject):
        result = pyqtSignal(bool, str, int)

    def __init__(self, func: Callable[[str], Tuple[bool, str, int]], args: str):
        super(EditFuncRunnable, self).__init__()
        self.setAutoDelete(True)
        self.signals = EditFuncRunnable.Signals()
        self.func = self.__wrap_edit_function(func)
        self.args = args

    def run(self):
        o0, o1, o2 = self.func(self.args)
        self.signals.result.emit(o0, o1, o2)
        self.signals.deleteLater()

    @staticmethod
    def __wrap_edit_function(func: Callable[[str], Tuple[bool, str, int]]):
        if func:
            return lambda text: func(os.path.expanduser(text) if text.startswith("~") else text)
        else:
            return func


class IndicatorLineEdit(QWidget):
    textChanged = pyqtSignal(str)

    def __init__(
        self,
        text: str = "",
        placeholder: str = "",
        completer: QCompleter = None,
        edit_func: Callable[[str], Tuple[bool, str, int]] = None,
        save_func: Callable[[str], None] = None,
        horiz_policy: QSizePolicy.Policy = QSizePolicy.Expanding,
        parent=None,
    ):
        super(IndicatorLineEdit, self).__init__(parent=parent)
        self.setObjectName(type(self).__name__)
        layout = QHBoxLayout(self)
        layout.setObjectName(f"{self.objectName()}Layout")
        layout.setContentsMargins(0, 0, 0, 0)
        # Add line_edit
        self.line_edit = QLineEdit(self)
        self.line_edit.setObjectName(f"{type(self).__name__}Edit")
        self.line_edit.setPlaceholderText(placeholder if placeholder else self.tr("Use global/default setting"))
        self.line_edit.setToolTip(placeholder if placeholder else "")
        self.line_edit.setSizePolicy(horiz_policy, QSizePolicy.Fixed)
        # Add completer
        self.setCompleter(completer)
        layout.addWidget(self.line_edit)
        if edit_func is not None:
            self.indicator_label = QLabel(self)
            self.indicator_label.setPixmap(qta_icon("ei.info-circle", color="gray").pixmap(16, 16))
            self.indicator_label.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
            layout.addWidget(self.indicator_label)

        self.__reasons = IndicatorReasonsStrings(self)

        self.__threadpool = QThreadPool(self)
        self.__threadpool.setMaxThreadCount(1)
        self.__thread: Optional[EditFuncRunnable] = None

        self.is_valid = False
        self.edit_func = edit_func
        self.save_func = save_func
        if text:
            self.line_edit.setText(text)
        self.line_edit.textChanged.connect(self.__edit)
        if self.edit_func is None:
            self.line_edit.textChanged.connect(self.__save)

        # lk: this can be placed here to trigger __edit
        # lk: it is going to save the input again if it is valid which
        # lk: is ok to do given the checks don't misbehave (they shouldn't)
        # lk: however it is going to edit any "understood" bad input to good input
        # lk: and we might not want that (but the validity check reports on the edited string)
        # lk: it is also going to trigger this widget's textChanged signal but that gets lost
        # if text:
        #     self.line_edit.setText(text)

    def deleteLater(self) -> None:
        if self.__thread is not None:
            self.__thread.signals.result.disconnect()
        super(IndicatorLineEdit, self).deleteLater()

    def text(self) -> str:
        return self.line_edit.text()

    def setText(self, text: str):
        self.line_edit.setText(text)

    def setCompleter(self, completer: Optional[QCompleter]):
        if old := self.line_edit.completer():
            old.deleteLater()
        if not completer:
            self.line_edit.setCompleter(None)
            return
        completer.popup().setItemDelegate(QStyledItemDelegate(self))
        completer.popup().setAlternatingRowColors(True)
        self.line_edit.setCompleter(completer)

    def extend_reasons(self, reasons: Dict):
        self.__reasons.extend(reasons)

    def __indicator(self, valid, reason: int = 0):
        color = "green" if valid else "red"
        self.indicator_label.setPixmap(qta_icon("ei.info-circle", color=color).pixmap(16, 16))
        if not valid:
            self.indicator_label.setToolTip(self.__reasons[reason])
        else:
            self.indicator_label.setToolTip(self.__reasons[IndicatorReasonsCommon.VALID])

    @pyqtSlot(bool, str, int)
    def __edit_handler(self, is_valid: bool, text: str, reason: int):
        self.__thread = None
        self.line_edit.blockSignals(True)
        if text != self.line_edit.text():
            self.line_edit.setText(text)
        self.line_edit.blockSignals(False)
        self.__indicator(is_valid, reason)
        if is_valid:
            self.__save(text)
        self.is_valid = is_valid
        self.textChanged.emit(text)

    def __edit(self, text):
        if self.edit_func is not None:
            if self.__thread is not None:
                self.__thread.signals.result.disconnect(self.__edit_handler)
            self.__thread = EditFuncRunnable(self.edit_func, text)
            self.__thread.signals.result.connect(self.__edit_handler)
            self.__threadpool.start(self.__thread)

    def __save(self, text):
        if self.save_func is not None:
            self.save_func(text)


class PathEditIconProvider(QFileIconProvider):
    icons = [
        ("mdi.file-cancel", "fa.file-excel-o"),  # Unknown
        ("mdi.desktop-classic", "fa.desktop"),  # Computer
        ("mdi.desktop-mac", "fa.desktop"),  # Desktop
        ("mdi.trash-can", "fa.trash"),  # Trashcan
        ("mdi.server-network", "fa.server"),  # Network
        ("mdi.harddisk", "fa.desktop"),  # Drive
        ("mdi.folder", "fa.folder"),  # Folder
        ("mdi.file", "fa.file"),  # File
        ("mdi.cog", "fa.cog"),  # Executable
    ]

    def __init__(self):
        super(PathEditIconProvider, self).__init__()
        self.icon_types = {}
        for idx, (icn, fallback) in enumerate(PathEditIconProvider.icons):
            self.icon_types.update({idx - 1: qta_icon(icn, fallback, color="#eeeeee")})

    def icon(self, info_type):
        if isinstance(info_type, QFileInfo):
            if info_type.isRoot():
                return self.icon_types[4]
            if info_type.isDir():
                return self.icon_types[5]
            if info_type.isFile():
                return self.icon_types[6]
            if info_type.isExecutable():
                return self.icon_types[7]
            return self.icon_types[-1]
        return self.icon_types[int(info_type)]


class PathEdit(IndicatorLineEdit):
    def __init__(
        self,
        path: str = "",
        file_mode: QFileDialog.FileMode = QFileDialog.AnyFile,
        file_filter: QDir.Filters = 0,
        name_filters: List[str] = None,
        placeholder: str = "",
        edit_func: Callable[[str], Tuple[bool, str, int]] = None,
        save_func: Callable[[str], None] = None,
        horiz_policy: QSizePolicy.Policy = QSizePolicy.Expanding,
        parent=None,
    ):
        self.__root_path = path if path else os.path.expanduser("~/")
        self.__completer = QCompleter()
        self.__completer_model = QFileSystemModel(self.__completer)
        try:
            self.__completer_model.setOptions(
                QFileSystemModel.DontWatchForChanges
                | QFileSystemModel.DontResolveSymlinks
                | QFileSystemModel.DontUseCustomDirectoryIcons
            )
        except AttributeError as e:  # Error on Ubuntu
            logger.warning(e)
        self.__completer_model.setIconProvider(PathEditIconProvider())
        self.__completer_model.setRootPath(path)
        if file_filter:
            self.__completer_model.setFilter(file_filter)
        if name_filters:
            self.__completer_model.setNameFilters(name_filters)
        self.__completer.setModel(self.__completer_model)

        super(PathEdit, self).__init__(
            text=path,
            placeholder=placeholder,
            completer=self.__completer,
            edit_func=edit_func,
            save_func=save_func,
            horiz_policy=horiz_policy,
            parent=parent,
        )
        self.setObjectName(type(self).__name__)
        self.line_edit.setMinimumSize(QSize(250, 0))
        self.path_select = QToolButton(self)
        self.path_select.setObjectName(f"{type(self).__name__}Button")
        layout = self.layout()
        layout.addWidget(self.path_select)

        self.path_select.setText(self.tr("Browse..."))

        self.__file_mode = file_mode
        self.__file_filter = file_filter
        self.__name_filter = name_filters

        self.path_select.clicked.connect(self.__set_path)

    def set_root(self, path: str):
        self.__root_path = path
        self.__completer_model.setRootPath(path)

    def __set_path(self):
        dlg_path = self.line_edit.text()
        if not dlg_path:
            dlg_path = self.__root_path
        dlg = QFileDialog(self, self.tr("Choose path"), dlg_path)
        dlg.setOption(QFileDialog.DontUseCustomDirectoryIcons)
        dlg.setIconProvider(PathEditIconProvider())
        dlg.setFileMode(self.__file_mode)
        if self.__file_mode == QFileDialog.Directory:
            dlg.setOption(QFileDialog.ShowDirsOnly, True)
        if self.__file_filter:
            dlg.setFilter(self.__file_filter)
        if self.__name_filter:
            dlg.setNameFilter(" ".join(self.__name_filter))
        if dlg.exec_():
            names = dlg.selectedFiles()
            self.line_edit.setText(names[0])
            self.__completer_model.setRootPath(names[0])



from typing import Optional, Tuple, List

from PyQt5.QtCore import Qt, QEvent, QObject
from PyQt5.QtGui import QPainter, QPixmap, QFontMetrics, QImage, QBrush, QColor, QShowEvent
from PyQt5.QtWidgets import QLabel

from rare.widgets.image_widget import ImageWidget


class ProgressLabel(QLabel):
    def __init__(self, parent=None):
        super(ProgressLabel, self).__init__(parent=parent)
        if self.parent() is not None:
            self.parent().installEventFilter(self)
        self.setObjectName(type(self).__name__)
        self.setAlignment(Qt.AlignHCenter | Qt.AlignVCenter)
        self.setFrameStyle(QLabel.StyledPanel)

    def __center_on_parent(self):
        fm = QFontMetrics(self.font())
        rect = fm.boundingRect(f"  {self.text()}  ")
        rect.moveCenter(self.parent().contentsRect().center())
        self.setGeometry(rect)

    def event(self, e: QEvent) -> bool:
        if e.type() == QEvent.ParentAboutToChange:
            if self.parent() is not None:
                self.parent().removeEventFilter(self)
        if e.type() == QEvent.ParentChange:
            if self.parent() is not None:
                self.parent().installEventFilter(self)
        return super().event(e)

    def showEvent(self, a0: QShowEvent) -> None:
        if a0.spontaneous():
            return super().showEvent(a0)
        self.__center_on_parent()
        super().showEvent(a0)

    def eventFilter(self, a0: QObject, a1: QEvent) -> bool:
        if a0 is self.parent() and a1.type() == QEvent.Resize:
            self.__center_on_parent()
            return a0.event(a1)
        return False

    @staticmethod
    def calculateColors(image: QImage) -> Tuple[QColor, QColor]:
        color: List[int] = [0, 0, 0]
        # take the two diagonals of the center square section
        min_d = min(image.width(), image.height())
        origin_w = (image.width() - min_d) // 2
        origin_h = (image.height() - min_d) // 2
        for x, y in zip(range(origin_w, min_d), range(origin_h, min_d)):
            pixel = image.pixelColor(x, y).getRgb()
            color = list(map(lambda t: sum(t) // 2, zip(pixel[0:3], color)))
        # take the V component of the HSV color
        fg_color = QColor(0, 0, 0) if QColor(*color).value() < 127 else QColor(255, 255, 255)
        bg_color = QColor(*map(lambda c: 255 - c, color))
        return bg_color, fg_color

    def setStyleSheetColors(self, bg: QColor, fg: QColor, brd: QColor):
        sheet = (
            f"QLabel#{type(self).__name__} {{"
            f"background-color: rgba({bg.red()}, {bg.green()}, {bg.blue()}, 65%);"
            f"color: rgb({fg.red()}, {fg.green()}, {fg.blue()});"
            f"border-color: rgb({brd.red()}, {brd.green()}, {brd.blue()});"
            f"}}"
        )
        self.setStyleSheet(sheet)


class LibraryWidget(ImageWidget):
    def __init__(self, parent=None) -> None:
        super(LibraryWidget, self).__init__(parent)
        self.progress_label = ProgressLabel(self)
        self.progress_label.setVisible(False)

        self._color_pixmap: Optional[QPixmap] = None
        self._gray_pixmap: Optional[QPixmap] = None
        # lk: keep percentage to not over-generate the image
        self._progress: int = -1

    def progressPixmap(self, color: QPixmap, gray: QPixmap, progress: int) -> QPixmap:
        """
        Paints the color image over the gray images based on progress percentage

        @param color:
        @param gray:
        @param progress:
        @return:
        """

        device = QPixmap(color.size())
        painter = QPainter(device)
        painter.setRenderHint(QPainter.SmoothPixmapTransform, self._smooth_transform)
        painter.setCompositionMode(QPainter.CompositionMode_Source)
        # lk: Vertical loading
        # prog_h = (device.height() * progress // 100)
        # brush = QBrush(gray)
        # painter.fillRect(device.rect().adjusted(0, 0, 0, -prog_h), brush)
        # brush.setTexture(color)
        # painter.fillRect(device.rect().adjusted(0, device.height() - prog_h, 0, 0), brush)
        # lk: Horizontal loading
        prog_w = device.width() * progress // 100
        brush = QBrush(gray)
        painter.fillRect(device.rect().adjusted(prog_w, 0, 0, 0), brush)
        brush.setTexture(color)
        painter.fillRect(device.rect().adjusted(0, 0, prog_w - device.width(), 0), brush)
        painter.end()
        device.setDevicePixelRatio(color.devicePixelRatioF())
        return device

    def showProgress(self, color_pm: QPixmap, gray_pm: QPixmap) -> None:
        self._color_pixmap = color_pm
        self._gray_pixmap = gray_pm
        bg_color, fg_color = self.progress_label.calculateColors(color_pm.toImage())
        self.progress_label.setStyleSheetColors(bg_color, fg_color, fg_color)
        self.progress_label.setVisible(True)
        self.updateProgress(0)

    def updateProgress(self, progress: int):
        self.progress_label.setText(f"{progress:02}%")
        if progress > self._progress:
            self._progress = progress
            self.setPixmap(self.progressPixmap(self._color_pixmap, self._gray_pixmap, progress))

    def hideProgress(self, stopped: bool):
        self._color_pixmap = None
        self._gray_pixmap = None
        self.progress_label.setVisible(stopped)
        self._progress = -1



import os
from getpass import getuser
from logging import getLogger

from PyQt5.QtCore import pyqtSignal
from PyQt5.QtWidgets import QFrame, QFileDialog
from legendary.core import LegendaryCore
from legendary.lfs.wine_helpers import get_shell_folders, read_registry

from rare.ui.components.dialogs.login.import_login import Ui_ImportLogin

logger = getLogger("ImportLogin")


class ImportLogin(QFrame):
    success = pyqtSignal()
    changed = pyqtSignal()

    # FIXME: Use pathspec instead of duplicated code
    if os.name == "nt":
        localappdata = os.path.expandvars("%LOCALAPPDATA%")
    else:
        localappdata = os.path.join("drive_c/users", getuser(), "Local Settings/Application Data")
    egl_appdata = os.path.join(localappdata, "EpicGamesLauncher", "Saved", "Config", "Windows")
    found = False

    def __init__(self, core: LegendaryCore, parent=None):
        super(ImportLogin, self).__init__(parent=parent)
        self.setFrameStyle(self.StyledPanel)
        self.ui = Ui_ImportLogin()
        self.ui.setupUi(self)

        self.core = core

        self.text_egl_found = self.tr("Found EGL Program Data. Click 'Next' to import them.")
        self.text_egl_notfound = self.tr("Could not find EGL Program Data. ")

        if os.name == "nt":
            if not self.core.egl.appdata_path and os.path.exists(self.egl_appdata):
                self.core.egl.appdata_path = self.egl_appdata
            if not self.core.egl.appdata_path:
                self.ui.status_label.setText(self.text_egl_notfound)
            else:
                self.ui.status_label.setText(self.text_egl_found)
                self.found = True
        else:
            if programdata_path := self.core.egl.programdata_path:
                if wine_pfx := programdata_path.split("drive_c")[0]:
                    self.ui.prefix_combo.addItem(wine_pfx)
            prefixes = self.get_wine_prefixes()
            if len(prefixes):
                self.ui.prefix_combo.addItems(prefixes)
                self.ui.status_label.setText(self.tr("Select the Wine prefix you want to import."))
            else:
                self.ui.status_label.setText(self.text_egl_notfound)

        self.ui.prefix_tool.clicked.connect(self.prefix_path)
        self.ui.prefix_combo.editTextChanged.connect(self.changed.emit)

    def get_wine_prefixes(self):
        possible_prefixes = [
            os.path.expanduser("~/.wine"),
            os.path.expanduser("~/Games/epic-games-store"),
        ]
        prefixes = []
        for prefix in possible_prefixes:
            if os.path.exists(os.path.join(prefix, self.egl_appdata)):
                prefixes.append(prefix)
        return prefixes

    def prefix_path(self):
        prefix_dialog = QFileDialog(self, self.tr("Choose path"), os.path.expanduser("~/"))
        prefix_dialog.setFileMode(QFileDialog.DirectoryOnly)
        if prefix_dialog.exec_():
            names = prefix_dialog.selectedFiles()
            self.ui.prefix_combo.setCurrentText(names[0])

    def is_valid(self) -> bool:
        if os.name == "nt":
            return self.found
        else:
            egl_wine_pfx = self.ui.prefix_combo.currentText()
            try:
                wine_folders = get_shell_folders(read_registry(egl_wine_pfx), egl_wine_pfx)
                self.egl_appdata = os.path.realpath(
                    os.path.join(wine_folders['Local AppData'], 'EpicGamesLauncher', 'Saved', 'Config', 'Windows'))
                if path_exists := os.path.exists(self.egl_appdata):
                    self.ui.status_label.setText(self.text_egl_found)
                return path_exists
            except KeyError:
                return False

    def do_login(self):
        self.ui.status_label.setText(self.tr("Loading..."))
        if os.name == "nt":
            pass
        else:
            logger.info("Using EGL appdata path at %s", {self.egl_appdata})
            self.core.egl.appdata_path = self.egl_appdata
        try:
            if self.core.auth_import():
                logger.info("Logged in as %s", {self.core.lgd.userdata['displayName']})
                self.success.emit()
            else:
                self.ui.status_label.setText(self.tr("Login failed."))
                logger.warning("Failed to import existing session.")
        except Exception as e:
            self.ui.status_label.setText(self.tr("Login failed. {}").format(str(e)))
            logger.warning("Failed to import existing session: %s", e)



from typing import Tuple, List, Union, Optional

from PyQt5.QtCore import QObject, pyqtSlot
from PyQt5.QtWidgets import QWidget

from rare.lgndr.core import LegendaryCore
from rare.models.game import RareGame
from rare.models.signals import GlobalSignals
from rare.shared import RareCore
from .icon_game_widget import IconGameWidget
from .list_game_widget import ListGameWidget


class LibraryWidgetController(QObject):
    def __init__(self, icon_container: QWidget, list_container: QWidget, parent: QWidget = None):
        super(LibraryWidgetController, self).__init__(parent=parent)
        self._icon_container: QWidget = icon_container
        self._list_container: QWidget = list_container
        self.rcore = RareCore.instance()
        self.core: LegendaryCore = self.rcore.core()
        self.signals: GlobalSignals = self.rcore.signals()

        self.signals.game.installed.connect(self.sort_list)
        self.signals.game.uninstalled.connect(self.sort_list)

    def add_game(self, rgame: RareGame):
        return self.add_widgets(rgame)

    def add_widgets(self, rgame: RareGame) -> Tuple[IconGameWidget, ListGameWidget]:
        icon_widget = IconGameWidget(rgame, self._icon_container)
        list_widget = ListGameWidget(rgame, self._list_container)
        return icon_widget, list_widget

    @staticmethod
    def __visibility(widget: Union[IconGameWidget,ListGameWidget], filter_name, search_text) -> Tuple[bool, float]:
        if filter_name == "hidden":
            visible = "hidden" in widget.rgame.metadata.tags
        elif "hidden" in widget.rgame.metadata.tags:
            visible = False
        elif filter_name == "installed":
            visible = widget.rgame.is_installed and not widget.rgame.is_unreal
        elif filter_name == "offline":
            visible = widget.rgame.can_run_offline and not widget.rgame.is_unreal
        elif filter_name == "32bit":
            visible = widget.rgame.is_win32 and not widget.rgame.is_unreal
        elif filter_name == "mac":
            visible = widget.rgame.is_mac and not widget.rgame.is_unreal
        elif filter_name == "installable":
            visible = not widget.rgame.is_non_asset and not widget.rgame.is_unreal
        elif filter_name == "include_ue":
            visible = True
        elif filter_name == "all":
            visible = not widget.rgame.is_unreal
        else:
            visible = True

        if (
            search_text not in widget.rgame.app_name.lower()
            and search_text not in widget.rgame.app_title.lower()
        ):
            opacity = 0.25
        else:
            opacity = 1.0

        return visible, opacity

    def filter_list(self, filter_name="all", search_text: str = ""):
        icon_widgets = self._icon_container.findChildren(IconGameWidget)
        list_widgets = self._list_container.findChildren(ListGameWidget)
        for iw in icon_widgets:
            visibility, opacity = self.__visibility(iw, filter_name, search_text)
            iw.setOpacity(opacity)
            iw.setVisible(visibility)
        for lw in list_widgets:
            visibility, opacity = self.__visibility(lw, filter_name, search_text)
            lw.setOpacity(opacity)
            lw.setVisible(visibility)
        self.sort_list(search_text)

    @pyqtSlot()
    def sort_list(self, sort_by: str = ""):
        # lk: this is the existing sorting implemenation
        # lk: it sorts by installed then by title
        if sort_by:
            self._icon_container.layout().sort(lambda x: (sort_by not in x.widget().rgame.app_title.lower(),))
        else:
            self._icon_container.layout().sort(
                key=lambda x: (
                # Sort by grant date
                #     x.widget().rgame.is_installed,
                #     not x.widget().rgame.is_non_asset,
                #     x.widget().rgame.grant_date(),
                # ), reverse=True
                    not x.widget().rgame.is_installed,
                    x.widget().rgame.is_non_asset,
                    x.widget().rgame.app_title,
                )
            )
        list_widgets = self._list_container.findChildren(ListGameWidget)
        if sort_by:
            list_widgets.sort(key=lambda x: (sort_by not in x.rgame.app_title.lower(),))
        else:
            list_widgets.sort(
                # Sort by grant date
                # key=lambda x: (x.rgame.is_installed, not x.rgame.is_non_asset, x.rgame.grant_date()), reverse=True
                key=lambda x: (not x.rgame.is_installed, x.rgame.is_non_asset, x.rgame.app_title)
            )
        for idx, wl in enumerate(list_widgets):
            self._list_container.layout().insertWidget(idx, wl)

    @pyqtSlot()
    @pyqtSlot(list)
    def update_list(self, app_names: List[str] = None):
        if not app_names:
            # lk: base it on icon widgets, the two lists should be identical
            icon_widgets = self._icon_container.findChildren(IconGameWidget)
            list_widgets = self._list_container.findChildren(ListGameWidget)
            icon_app_names = set([iw.rgame.app_name for iw in icon_widgets])
            list_app_names = set([lw.rgame.app_name for lw in list_widgets])
            games = list(self.rcore.games)
            game_app_names = set([g.app_name for g in games])
            new_icon_app_names = game_app_names.difference(icon_app_names)
            new_list_app_names = game_app_names.difference(list_app_names)
            for app_name in new_icon_app_names:
                game = self.rcore.get_game(app_name)
                iw = IconGameWidget(game)
                self._icon_container.layout().addWidget(iw)
            for app_name in new_list_app_names:
                game = self.rcore.get_game(app_name)
                lw = ListGameWidget(game)
                self._list_container.layout().addWidget(lw)
            self.sort_list()

    def __find_widget(self, app_name: str) -> Tuple[Union[IconGameWidget, None], Union[ListGameWidget, None]]:
        iw = self._icon_container.findChild(IconGameWidget, app_name)
        lw = self._list_container.findChild(ListGameWidget, app_name)
        return iw, lw



import os
import platform as pf
import shutil
from typing import Tuple, List, Union, Optional

from PyQt5.QtCore import Qt, QThreadPool, QSettings, QCoreApplication
from PyQt5.QtCore import pyqtSignal, pyqtSlot
from PyQt5.QtGui import QCloseEvent, QKeyEvent, QShowEvent
from PyQt5.QtWidgets import QDialog, QFileDialog, QCheckBox, QLayout, QWidget, QVBoxLayout, QFormLayout
from legendary.utils.selective_dl import get_sdl_appname

from rare.models.game import RareGame
from rare.models.install import InstallDownloadModel, InstallQueueItemModel, InstallOptionsModel
from rare.shared import LegendaryCoreSingleton, ArgumentsSingleton
from rare.shared.workers.install_info import InstallInfoWorker
from rare.ui.components.dialogs.install_dialog import Ui_InstallDialog
from rare.ui.components.dialogs.install_dialog_advanced import Ui_InstallDialogAdvanced
from rare.utils import config_helper
from rare.utils.misc import format_size
from rare.widgets.collapsible_widget import CollapsibleFrame
from rare.widgets.indicator_edit import PathEdit, IndicatorReasonsCommon


class InstallDialogAdvanced(CollapsibleFrame):
    def __init__(self, parent=None):
        widget = QWidget()
        title = widget.tr("Advanced options")
        self.ui = Ui_InstallDialogAdvanced()
        self.ui.setupUi(widget)
        super(InstallDialogAdvanced, self).__init__(widget=widget, title=title, parent=parent)


class InstallDialog(QDialog):
    result_ready = pyqtSignal(InstallQueueItemModel)

    def __init__(self, rgame: RareGame, options: InstallOptionsModel, parent=None):
        super(InstallDialog, self).__init__(parent=parent)
        self.ui = Ui_InstallDialog()
        self.ui.setupUi(self)
        self.setAttribute(Qt.WA_DeleteOnClose, True)
        self.setWindowFlags(Qt.Dialog | Qt.CustomizeWindowHint | Qt.WindowTitleHint)
        # lk: set object names for CSS properties
        self.ui.install_button.setObjectName("InstallButton")

        self.core = LegendaryCoreSingleton()
        self.rgame = rgame
        self.options = options
        self.__download: Optional[InstallDownloadModel] = None

        self.advanced = InstallDialogAdvanced(parent=self)
        self.ui.advanced_layout.addWidget(self.advanced)

        self.selectable = CollapsibleFrame(widget=None, title=self.tr("Optional downloads"), parent=self)
        self.ui.selectable_layout.addWidget(self.selectable)

        self.options_changed = False
        self.worker_running = False
        self.reject_close = True

        self.threadpool = QThreadPool(self)
        self.threadpool.setMaxThreadCount(1)

        if options.repair_mode:
            header = self.tr("Repair")
            if options.repair_and_update:
                header = self.tr("Repair and update")
        elif options.update:
            header = self.tr("Update")
        elif options.reset_sdl:
            header = self.tr("Modify")
        else:
            header = self.tr("Install")
        self.ui.install_dialog_label.setText(f'<h3>{header} "{self.rgame.app_title}"</h3>')
        self.setWindowTitle(f'{header} "{self.rgame.app_title}" - {QCoreApplication.instance().applicationName()}')

        if options.base_path:
            base_path = options.base_path
        elif rgame.is_installed:
            base_path = rgame.install_path
        else:
            base_path = self.core.get_default_install_dir(rgame.default_platform)

        self.install_dir_edit = PathEdit(
            path=base_path,
            file_mode=QFileDialog.DirectoryOnly,
            edit_func=self.option_changed,
            save_func=self.save_install_edit,
            parent=self,
        )
        self.ui.install_dialog_layout.setWidget(
            self.ui.install_dialog_layout.getWidgetPosition(self.ui.install_dir_label)[0],
            QFormLayout.FieldRole, self.install_dir_edit
        )

        self.install_dir_edit.setDisabled(rgame.is_installed)
        self.ui.install_dir_label.setDisabled(rgame.is_installed)
        self.ui.shortcut_label.setDisabled(rgame.is_installed)
        self.ui.shortcut_check.setDisabled(rgame.is_installed)
        self.ui.shortcut_check.setChecked(not rgame.is_installed and QSettings().value("create_shortcut", True, bool))

        self.error_box()

        self.ui.platform_combo.addItems(reversed(rgame.platforms))
        combo_text = rgame.igame.platform if rgame.is_installed else rgame.default_platform
        self.ui.platform_combo.setCurrentIndex(self.ui.platform_combo.findText(combo_text))
        self.ui.platform_combo.currentIndexChanged.connect(lambda i: self.option_changed(None))
        self.ui.platform_combo.currentIndexChanged.connect(self.check_incompatible_platform)
        self.ui.platform_combo.currentIndexChanged.connect(self.reset_install_dir)
        self.ui.platform_combo.currentIndexChanged.connect(self.reset_sdl_list)

        self.ui.platform_label.setDisabled(rgame.is_installed)
        self.ui.platform_combo.setDisabled(rgame.is_installed)

        self.advanced.ui.max_workers_spin.setValue(self.core.lgd.config.getint("Legendary", "max_workers", fallback=0))
        self.advanced.ui.max_workers_spin.valueChanged.connect(self.option_changed)

        self.advanced.ui.max_memory_spin.setValue(self.core.lgd.config.getint("Legendary", "max_memory", fallback=0))
        self.advanced.ui.max_memory_spin.valueChanged.connect(self.option_changed)

        self.advanced.ui.dl_optimizations_check.stateChanged.connect(self.option_changed)
        self.advanced.ui.force_download_check.stateChanged.connect(self.option_changed)
        self.advanced.ui.ignore_space_check.stateChanged.connect(self.option_changed)
        self.advanced.ui.download_only_check.stateChanged.connect(
            lambda: self.non_reload_option_changed("download_only")
        )
        self.ui.shortcut_check.stateChanged.connect(
            lambda: self.non_reload_option_changed("shortcut")
        )

        self.selectable_checks: List[TagCheckBox] = []
        self.config_tags: Optional[List[str]] = None

        self.reset_install_dir(self.ui.platform_combo.currentIndex())
        self.reset_sdl_list(self.ui.platform_combo.currentIndex())
        self.check_incompatible_platform(self.ui.platform_combo.currentIndex())

        self.ui.install_button.setEnabled(False)

        if self.options.overlay:
            self.ui.platform_label.setEnabled(False)
            self.ui.platform_combo.setEnabled(False)
            self.advanced.ui.ignore_space_label.setEnabled(False)
            self.advanced.ui.ignore_space_check.setEnabled(False)
            self.advanced.ui.download_only_label.setEnabled(False)
            self.advanced.ui.download_only_check.setEnabled(False)
            self.ui.shortcut_label.setEnabled(False)
            self.ui.shortcut_check.setEnabled(False)
            self.selectable.setEnabled(False)

        if pf.system() == "Darwin":
            self.ui.shortcut_label.setDisabled(True)
            self.ui.shortcut_check.setDisabled(True)
            self.ui.shortcut_check.setChecked(False)
            self.ui.shortcut_check.setToolTip(self.tr("Creating a shortcut is not supported on macOS"))

        self.advanced.ui.install_prereqs_label.setEnabled(False)
        self.advanced.ui.install_prereqs_check.setEnabled(False)
        self.advanced.ui.install_prereqs_check.stateChanged.connect(
            lambda: self.non_reload_option_changed("install_prereqs")
        )

        self.non_reload_option_changed("shortcut")

        self.ui.cancel_button.clicked.connect(self.__on_cancel)
        self.ui.verify_button.clicked.connect(self.__on_verify)
        self.ui.install_button.clicked.connect(self.__on_install)

        self.advanced.ui.install_prereqs_check.setChecked(self.options.install_prereqs)

        self.ui.install_dialog_layout.setSizeConstraint(QLayout.SetFixedSize)

    def showEvent(self, a0: QShowEvent) -> None:
        if a0.spontaneous():
            return super().showEvent(a0)
        self.save_install_edit(self.install_dir_edit.text())
        super().showEvent(a0)

    def execute(self):
        if self.options.silent:
            self.reject_close = False
            self.get_download_info()
        else:
            self.setModal(True)
            self.__on_verify()
            self.show()

    @pyqtSlot(int)
    def reset_install_dir(self, index: int):
        if not self.rgame.is_installed:
            platform = self.ui.platform_combo.itemText(index)
            default_dir = self.core.get_default_install_dir(platform)
            self.install_dir_edit.setText(default_dir)

    @pyqtSlot(int)
    def reset_sdl_list(self, index: int):
        platform = self.ui.platform_combo.itemText(index)
        for cb in self.selectable_checks:
            cb.disconnect()
            cb.deleteLater()
        self.selectable_checks.clear()

        if config_tags := self.core.lgd.config.get(self.rgame.app_name, 'install_tags', fallback=None):
            self.config_tags = config_tags.split(",")
        config_disable_sdl = self.core.lgd.config.getboolean(self.rgame.app_name, 'disable_sdl', fallback=False)
        sdl_name = get_sdl_appname(self.rgame.app_name)
        if not config_disable_sdl and sdl_name is not None:
            sdl_data = self.core.get_sdl_data(sdl_name, platform=platform)
            if sdl_data:
                widget = QWidget(self.selectable)
                layout = QVBoxLayout(widget)
                layout.setSpacing(0)
                for tag, info in sdl_data.items():
                    cb = TagCheckBox(info["name"], info["description"], info["tags"])
                    if tag == "__required":
                        cb.setChecked(True)
                        cb.setDisabled(True)
                    if self.config_tags is not None:
                        if all(elem in self.config_tags for elem in info["tags"]):
                            cb.setChecked(True)
                    layout.addWidget(cb)
                    self.selectable_checks.append(cb)
                for cb in self.selectable_checks:
                    cb.stateChanged.connect(self.option_changed)
                self.selectable.setWidget(widget)
        else:
            self.selectable.setDisabled(True)

    @pyqtSlot(int)
    def check_incompatible_platform(self, index: int):
        platform = self.ui.platform_combo.itemText(index)
        if platform == "Mac" and pf.system() != "Darwin":
            self.error_box(
                self.tr("Warning"),
                self.tr("You will not be able to run the game if you select <b>{}</b> as platform").format(platform)
            )
        else:
            self.error_box()

    def get_options(self):
        self.options.base_path = "" if self.rgame.is_installed else self.install_dir_edit.text()
        self.options.max_workers = self.advanced.ui.max_workers_spin.value()
        self.options.shared_memory = self.advanced.ui.max_memory_spin.value()
        self.options.order_opt = self.advanced.ui.dl_optimizations_check.isChecked()
        self.options.force = self.advanced.ui.force_download_check.isChecked()
        self.options.ignore_space = self.advanced.ui.ignore_space_check.isChecked()
        self.options.no_install = self.advanced.ui.download_only_check.isChecked()
        self.options.platform = self.ui.platform_combo.currentText()
        self.options.install_prereqs = self.advanced.ui.install_prereqs_check.isChecked()
        self.options.create_shortcut = self.ui.shortcut_check.isChecked()
        if self.selectable_checks:
            self.options.install_tag = [""]
            for cb in self.selectable_checks:
                if data := cb.isChecked():
                    # noinspection PyTypeChecker
                    self.options.install_tag.extend(data)

    def get_download_info(self):
        self.__download = None
        info_worker = InstallInfoWorker(self.core, self.options)
        info_worker.signals.result.connect(self.on_worker_result)
        info_worker.signals.failed.connect(self.on_worker_failed)
        info_worker.signals.finished.connect(self.on_worker_finished)
        self.worker_running = True
        self.threadpool.start(info_worker)

    def __on_verify(self):
        self.error_box()
        message = self.tr("Updating...")
        self.ui.download_size_text.setText(message)
        self.ui.download_size_text.setStyleSheet("font-style: italic; font-weight: normal")
        self.ui.install_size_text.setText(message)
        self.ui.install_size_text.setStyleSheet("font-style: italic; font-weight: normal")
        self.ui.cancel_button.setEnabled(False)
        self.ui.verify_button.setEnabled(False)
        self.ui.install_button.setEnabled(False)
        self.options_changed = False
        self.get_options()
        self.get_download_info()

    def option_changed(self, path) -> Tuple[bool, str, int]:
        self.options_changed = True
        self.ui.install_button.setEnabled(False)
        self.ui.verify_button.setEnabled(not self.worker_running)
        return True, path, IndicatorReasonsCommon.VALID

    def save_install_edit(self, path: str):
        if not os.path.exists(path):
            return
        _, _, free_space = shutil.disk_usage(path)
        self.ui.avail_space.setText(format_size(free_space))

    def non_reload_option_changed(self, option: str):
        if option == "download_only":
            self.options.no_install = self.advanced.ui.download_only_check.isChecked()
        elif option == "shortcut":
            QSettings().setValue("create_shortcut", self.ui.shortcut_check.isChecked())
            self.options.create_shortcut = self.ui.shortcut_check.isChecked()
        elif option == "install_prereqs":
            self.options.install_prereqs = self.advanced.ui.install_prereqs_check.isChecked()

    def __on_cancel(self):
        if self.config_tags is not None:
            config_helper.add_option(self.rgame.app_name, 'install_tags', ','.join(self.config_tags))
        else:
            # lk: this is purely for cleaning any install tags we might have added erroneously to the config
            config_helper.remove_option(self.rgame.app_name, 'install_tags')

        self.__download = None
        self.reject_close = False
        self.close()

    def __on_install(self):
        self.reject_close = False
        self.close()

    @staticmethod
    def same_platform(download: InstallDownloadModel) -> bool:
        platform = download.igame.platform
        if pf.system() == "Windows":
            return platform == "Windows" or platform == "Win32"
        elif pf.system() == "Darwin":
            return platform == "Mac"
        else:
            return False

    @pyqtSlot(InstallDownloadModel)
    def on_worker_result(self, download: InstallDownloadModel):
        self.__download = download
        download_size = download.analysis.dl_size
        install_size = download.analysis.install_size
        # install_size = self.dl_item.download.analysis.disk_space_delta
        if download_size or (not download_size and (download.game.is_dlc or download.repair)):
            self.ui.download_size_text.setText(format_size(download_size))
            self.ui.download_size_text.setStyleSheet("font-style: normal; font-weight: bold")
            self.ui.install_button.setEnabled(not self.options_changed)
        else:
            self.ui.install_size_text.setText(self.tr("Game already installed"))
            self.ui.install_size_text.setStyleSheet("font-style: italics; font-weight: normal")
        self.ui.install_size_text.setText(format_size(install_size))
        self.ui.install_size_text.setStyleSheet("font-style: normal; font-weight: bold")
        self.ui.verify_button.setEnabled(self.options_changed)
        self.ui.cancel_button.setEnabled(True)
        has_prereqs = bool(download.igame.prereq_info) and not download.igame.prereq_info.get("installed", False)
        if has_prereqs:
            prereq_name = download.igame.prereq_info.get("name", "")
            prereq_path = os.path.split(download.igame.prereq_info.get("path", ""))[-1]
            prereq_desc = prereq_name if prereq_name else prereq_path
            self.advanced.ui.install_prereqs_check.setText(self.tr("Also install: {}").format(prereq_desc))
        else:
            self.advanced.ui.install_prereqs_check.setText("")
        # Offer to install prerequisites only on same platforms
        self.advanced.ui.install_prereqs_label.setEnabled(has_prereqs)
        self.advanced.ui.install_prereqs_check.setEnabled(has_prereqs)
        self.advanced.ui.install_prereqs_check.setChecked(has_prereqs and self.same_platform(download))
        if self.options.silent:
            self.close()

    def on_worker_failed(self, message: str):
        error_text = self.tr("Error")
        self.ui.download_size_text.setText(error_text)
        self.ui.install_size_text.setText(error_text)
        self.error_box(error_text, message)
        self.ui.verify_button.setEnabled(self.options_changed)
        self.ui.cancel_button.setEnabled(True)
        if self.options.silent:
            self.show()

    def error_box(self, label: str = "", message: str = ""):
        self.ui.warning_label.setVisible(bool(label))
        self.ui.warning_label.setText(label)
        self.ui.warning_text.setVisible(bool(message))
        self.ui.warning_text.setText(message)

    def on_worker_finished(self):
        self.worker_running = False

    # lk: happens when close() is called, also when top right 'X' is pressed.
    # lk: reject any events not coming from the buttons in case the WM
    # lk: doesn't honor the window hints
    def closeEvent(self, a0: QCloseEvent) -> None:
        if self.reject_close:
            a0.ignore()
        else:
            self.threadpool.clear()
            self.threadpool.waitForDone()
            self.result_ready.emit(InstallQueueItemModel(options=self.options, download=self.__download))
            super(InstallDialog, self).closeEvent(a0)

    def keyPressEvent(self, e: QKeyEvent) -> None:
        if e.key() == Qt.Key_Escape:
            e.accept()
            self.__on_cancel()


class TagCheckBox(QCheckBox):
    def __init__(self, text, desc, tags: List[str], parent=None):
        super(TagCheckBox, self).__init__(parent)
        self.setText(text)
        self.setToolTip(desc)
        self.tags = tags

    def isChecked(self) -> Union[bool, List[str]]:
        return self.tags if super(TagCheckBox, self).isChecked() else False






# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/dialogs/install_dialog_advanced.ui'
#
# Created by: PyQt5 UI code generator 5.15.7
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_InstallDialogAdvanced(object):
    def setupUi(self, InstallDialogAdvanced):
        InstallDialogAdvanced.setObjectName("InstallDialogAdvanced")
        InstallDialogAdvanced.resize(379, 208)
        InstallDialogAdvanced.setWindowTitle("InstallDialogAdvanced")
        self.install_dialog_advanced_layout = QtWidgets.QFormLayout(InstallDialogAdvanced)
        self.install_dialog_advanced_layout.setObjectName("install_dialog_advanced_layout")
        self.max_workers_label = QtWidgets.QLabel(InstallDialogAdvanced)
        self.max_workers_label.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.max_workers_label.setObjectName("max_workers_label")
        self.install_dialog_advanced_layout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.max_workers_label)
        self.max_workers_layout = QtWidgets.QHBoxLayout()
        self.max_workers_layout.setObjectName("max_workers_layout")
        self.max_workers_spin = QtWidgets.QSpinBox(InstallDialogAdvanced)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.max_workers_spin.sizePolicy().hasHeightForWidth())
        self.max_workers_spin.setSizePolicy(sizePolicy)
        self.max_workers_spin.setObjectName("max_workers_spin")
        self.max_workers_layout.addWidget(self.max_workers_spin)
        self.max_workers_info = QtWidgets.QLabel(InstallDialogAdvanced)
        font = QtGui.QFont()
        font.setItalic(True)
        self.max_workers_info.setFont(font)
        self.max_workers_info.setObjectName("max_workers_info")
        self.max_workers_layout.addWidget(self.max_workers_info)
        self.install_dialog_advanced_layout.setLayout(0, QtWidgets.QFormLayout.FieldRole, self.max_workers_layout)
        self.max_memory_label = QtWidgets.QLabel(InstallDialogAdvanced)
        self.max_memory_label.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.max_memory_label.setObjectName("max_memory_label")
        self.install_dialog_advanced_layout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.max_memory_label)
        self.max_memory_layout = QtWidgets.QHBoxLayout()
        self.max_memory_layout.setObjectName("max_memory_layout")
        self.max_memory_spin = QtWidgets.QSpinBox(InstallDialogAdvanced)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.max_memory_spin.sizePolicy().hasHeightForWidth())
        self.max_memory_spin.setSizePolicy(sizePolicy)
        self.max_memory_spin.setMinimum(0)
        self.max_memory_spin.setMaximum(10240)
        self.max_memory_spin.setSingleStep(128)
        self.max_memory_spin.setProperty("value", 1024)
        self.max_memory_spin.setObjectName("max_memory_spin")
        self.max_memory_layout.addWidget(self.max_memory_spin)
        self.max_memory_info = QtWidgets.QLabel(InstallDialogAdvanced)
        font = QtGui.QFont()
        font.setItalic(True)
        self.max_memory_info.setFont(font)
        self.max_memory_info.setObjectName("max_memory_info")
        self.max_memory_layout.addWidget(self.max_memory_info)
        self.install_dialog_advanced_layout.setLayout(1, QtWidgets.QFormLayout.FieldRole, self.max_memory_layout)
        self.install_prereqs_label = QtWidgets.QLabel(InstallDialogAdvanced)
        self.install_prereqs_label.setObjectName("install_prereqs_label")
        self.install_dialog_advanced_layout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.install_prereqs_label)
        self.install_prereqs_check = QtWidgets.QCheckBox(InstallDialogAdvanced)
        font = QtGui.QFont()
        font.setItalic(True)
        self.install_prereqs_check.setFont(font)
        self.install_prereqs_check.setText("")
        self.install_prereqs_check.setChecked(False)
        self.install_prereqs_check.setObjectName("install_prereqs_check")
        self.install_dialog_advanced_layout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.install_prereqs_check)
        self.dl_optimizations_label = QtWidgets.QLabel(InstallDialogAdvanced)
        self.dl_optimizations_label.setObjectName("dl_optimizations_label")
        self.install_dialog_advanced_layout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.dl_optimizations_label)
        self.dl_optimizations_check = QtWidgets.QCheckBox(InstallDialogAdvanced)
        self.dl_optimizations_check.setText("")
        self.dl_optimizations_check.setChecked(False)
        self.dl_optimizations_check.setObjectName("dl_optimizations_check")
        self.install_dialog_advanced_layout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.dl_optimizations_check)
        self.force_download_label = QtWidgets.QLabel(InstallDialogAdvanced)
        self.force_download_label.setObjectName("force_download_label")
        self.install_dialog_advanced_layout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.force_download_label)
        self.force_download_check = QtWidgets.QCheckBox(InstallDialogAdvanced)
        self.force_download_check.setText("")
        self.force_download_check.setObjectName("force_download_check")
        self.install_dialog_advanced_layout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.force_download_check)
        self.ignore_space_label = QtWidgets.QLabel(InstallDialogAdvanced)
        self.ignore_space_label.setObjectName("ignore_space_label")
        self.install_dialog_advanced_layout.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.ignore_space_label)
        self.ignore_space_check = QtWidgets.QCheckBox(InstallDialogAdvanced)
        font = QtGui.QFont()
        font.setItalic(True)
        self.ignore_space_check.setFont(font)
        self.ignore_space_check.setObjectName("ignore_space_check")
        self.install_dialog_advanced_layout.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.ignore_space_check)
        self.download_only_label = QtWidgets.QLabel(InstallDialogAdvanced)
        self.download_only_label.setObjectName("download_only_label")
        self.install_dialog_advanced_layout.setWidget(6, QtWidgets.QFormLayout.LabelRole, self.download_only_label)
        self.download_only_check = QtWidgets.QCheckBox(InstallDialogAdvanced)
        font = QtGui.QFont()
        font.setItalic(True)
        self.download_only_check.setFont(font)
        self.download_only_check.setObjectName("download_only_check")
        self.install_dialog_advanced_layout.setWidget(6, QtWidgets.QFormLayout.FieldRole, self.download_only_check)

        self.retranslateUi(InstallDialogAdvanced)

    def retranslateUi(self, InstallDialogAdvanced):
        _translate = QtCore.QCoreApplication.translate
        self.max_workers_label.setText(_translate("InstallDialogAdvanced", "Max workers"))
        self.max_workers_info.setText(_translate("InstallDialogAdvanced", "Less is slower. (0: Default)"))
        self.max_memory_label.setText(_translate("InstallDialogAdvanced", "Max shared memory"))
        self.max_memory_spin.setSuffix(_translate("InstallDialogAdvanced", "MiB"))
        self.max_memory_info.setText(_translate("InstallDialogAdvanced", "Less is slower (0: Default)"))
        self.install_prereqs_label.setText(_translate("InstallDialogAdvanced", "Install prerequisites"))
        self.dl_optimizations_label.setText(_translate("InstallDialogAdvanced", "Enable reordering"))
        self.force_download_label.setText(_translate("InstallDialogAdvanced", "Force redownload"))
        self.ignore_space_label.setText(_translate("InstallDialogAdvanced", "Ignore free space"))
        self.ignore_space_check.setText(_translate("InstallDialogAdvanced", "Use with caution!"))
        self.download_only_label.setText(_translate("InstallDialogAdvanced", "Download only"))
        self.download_only_check.setText(_translate("InstallDialogAdvanced", "Do not try to install."))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    InstallDialogAdvanced = QtWidgets.QWidget()
    ui = Ui_InstallDialogAdvanced()
    ui.setupUi(InstallDialogAdvanced)
    InstallDialogAdvanced.show()
    sys.exit(app.exec_())
